
#include	"applicationGlobals.h"

#define kFinderSig		'FNDR'
#define kAEFinderEvents	'FNDR'
#define kSystemType	'MACS'

#define kAEOpenSelection	'sope'
#define keySelection		'fsel'

#ifdef TARGET_API_MAC_CARBON
static AEEventHandlerUPP doOpenAppHandler;
static AEEventHandlerUPP doOpenDocHandler;
static AEEventHandlerUPP doPrintDocHandler;
static AEEventHandlerUPP doQuitAppHandler;
static AEEventHandlerUPP doShowPrefHandler;
#endif

#ifdef TARGET_API_MAC_CARBON
extern Boolean	gDone;
#endif

OSErr findAProcess(OSType, OSType, ProcessSerialNumber*);

// Given a FSSpecPtr to either an application or a document, openSelection creates a
// finder Open Selection Apple event for the object described by the FSSpec.
OSErr openSelection(FSSpecPtr theDoc)
{
	AppleEvent	aeEvent;			// the event to create;
	AEDesc		myAddressDesc;	// descriptors
	AEDesc		aeDirDesc;
	AEDesc		listElem;
	AEDesc		fileList;			// our list
	FSSpec		dirSpec;
	AliasHandle	dirAlias;			// alias to directory with our file
	AliasHandle	fileAlias;			// alias of the file itself
	ProcessSerialNumber process;		// the finder's psn
	OSErr		iErr;	

	// Get the psn of the Finder and create the target address
	if(findAProcess(kFinderSig,kSystemType,&process))
			return procNotFound;
	iErr = AECreateDesc(typeProcessSerialNumber,(Ptr) &process,sizeof(process), &myAddressDesc);
	if(iErr!=noErr)	
		return iErr;

	// Create an empty 
	iErr = AECreateAppleEvent (kAEFinderEvents, kAEOpenSelection,
					&myAddressDesc, kAutoGenerateReturnID, kAnyTransactionID, &aeEvent);
	if(iErr!=noErr)	
		return iErr;

	// Make an FSSpec and alias for the parent folder, and an alias for the file
	FSMakeFSSpec(theDoc->vRefNum,theDoc->parID,nil,&dirSpec);
	NewAlias(nil,&dirSpec,&dirAlias);
	NewAlias(nil,theDoc,&fileAlias);

	// Create the file list.
	iErr=AECreateList(nil,0,false,&fileList);
	if(iErr!=noErr)	
		return iErr;

	// Create the folder descriptor
	HLock((Handle)dirAlias);
	AECreateDesc(typeAlias, (Ptr) *dirAlias, GetHandleSize((Handle) dirAlias), &aeDirDesc);
	HUnlock((Handle)dirAlias);
	DisposeHandle((Handle)dirAlias);

	iErr = AEPutParamDesc(&aeEvent,keyDirectObject,&aeDirDesc);
	if(iErr == noErr)
	{
		AEDisposeDesc(&aeDirDesc);
		HLock((Handle)fileAlias);
		AECreateDesc(typeAlias, (Ptr)*fileAlias,GetHandleSize((Handle)fileAlias), &listElem);
		HUnlock((Handle)fileAlias);
		DisposeHandle((Handle)fileAlias);
		iErr = AEPutDesc(&fileList,0,&listElem);
	}
	if(iErr!=noErr)		
		return iErr;
	AEDisposeDesc(&listElem);

	iErr = AEPutParamDesc(&aeEvent,keySelection,&fileList);
	if(iErr!=noErr)		
		return iErr;

	iErr = AEDisposeDesc(&fileList);

	iErr = AESend(&aeEvent, nil,	kAENoReply+kAEAlwaysInteract+kAECanSwitchLayer,
			kAENormalPriority, kAEDefaultTimeout, nil, nil);

	AEDisposeDesc(&aeEvent);
	AEDisposeDesc(&myAddressDesc);
	
	return(iErr);
}

// Search through the current process list to find the given application.
// Using the Process Manager for a similar way of doing this.
OSErr findAProcess(OSType typeToFind, OSType creatorToFind,ProcessSerialNumberPtr processSN)
{
	ProcessInfoRec	tempInfo;
	FSSpec		procSpec;
	Str31		processName;
	OSErr		myErr = noErr;

	// start at the beginning of the process list
	processSN->lowLongOfPSN = kNoProcess;
	processSN->highLongOfPSN = kNoProcess;

	// initialize the process information record
	tempInfo.processInfoLength = sizeof(ProcessInfoRec);
	tempInfo.processName = (StringPtr)&processName;
	tempInfo.processAppSpec = &procSpec;

	while((tempInfo.processSignature != creatorToFind ||
		tempInfo.processType != typeToFind) ||
		myErr != noErr)
	{
		myErr = GetNextProcess(processSN);
		if (myErr == noErr)
			GetProcessInformation(processSN, &tempInfo);
	}

	return(myErr);
}

#pragma mark -

void	installRequiredAppleEvent(void)
{
	OSErr	err;
	long		feature;

	err = Gestalt(gestaltAppleEventsAttr, &feature);
	
	if (err != noErr)
		return;
	else
	{
		if (!(feature & (kGestaltMask<<gestaltAppleEventsPresent)))
		{
			alertUser(eNoAppleEvent);
			return;
		}
	}

#ifdef TARGET_API_MAC_CARBON
        doOpenAppHandler = NewAEEventHandlerUPP((AEEventHandlerProcPtr)doOpenApp);
	err = AEInstallEventHandler(kCoreEventClass, kAEOpenApplication, 
		doOpenAppHandler, 0L, false);
#else
	err = AEInstallEventHandler(kCoreEventClass, kAEOpenApplication, 
		NewAEEventHandlerProc(doOpenApp), 0L, false);
#endif
	if (err != noErr) 
		alertUser(eNoAppleEvent);
	
#ifdef TARGET_API_MAC_CARBON
        doOpenDocHandler = NewAEEventHandlerUPP((AEEventHandlerProcPtr)doOpenDoc);
	err = AEInstallEventHandler(kCoreEventClass, kAEOpenDocuments, 
		doOpenDocHandler, 0L, false);
#else
	err = AEInstallEventHandler(kCoreEventClass, kAEOpenDocuments, 
		NewAEEventHandlerProc(doOpenDoc), 0L, false);
#endif
	if (err != noErr) 
		alertUser(eNoAppleEvent);
	
#ifdef TARGET_API_MAC_CARBON
        doPrintDocHandler = NewAEEventHandlerUPP((AEEventHandlerProcPtr)doPrintDoc);
	err = AEInstallEventHandler(kCoreEventClass, kAEPrintDocuments, 
		doPrintDocHandler, 0L, false);
#else
	err = AEInstallEventHandler(kCoreEventClass, kAEPrintDocuments, 
		NewAEEventHandlerProc(doPrintDoc), 0L, false);
#endif
	if (err != noErr) 
		alertUser(eNoAppleEvent);
	
#ifdef TARGET_API_MAC_CARBON
        doQuitAppHandler = NewAEEventHandlerUPP((AEEventHandlerProcPtr)doQuitApp);
	err = AEInstallEventHandler(kCoreEventClass, kAEQuitApplication, 
		doQuitAppHandler, 0L, false);
#else
	err = AEInstallEventHandler(kCoreEventClass, kAEQuitApplication, 
		NewAEEventHandlerProc(doQuitApp), 0L, false);
#endif
	if (err != noErr) 
		alertUser(eNoAppleEvent);
                
#ifdef TARGET_API_MAC_CARBON
        doShowPrefHandler = NewAEEventHandlerUPP((AEEventHandlerProcPtr)doShowPref);
	err = AEInstallEventHandler(kCoreEventClass, kAEShowPreferences, 
		doShowPrefHandler, 0L, false);
	if (err != noErr) 
		alertUser(eNoAppleEvent);
#endif
}

#ifdef TARGET_API_MAC_CARBON
pascal OSErr doOpenApp(const AppleEvent *theAppleEvent, AppleEvent *reply, UInt32 refCon)
#else
pascal	OSErr	doOpenApp(AppleEvent *theAppleEvent, AppleEvent *reply, long refCon)
#endif
{	
	#pragma unused(theAppleEvent, reply, refCon)

	OSErr	iErr = noErr;

	checkPrefDialog();		//if there is pref dialog, bring it front

	return(iErr);	
}

#ifdef TARGET_API_MAC_CARBON
pascal OSErr doOpenDoc(const AppleEvent *theAppleEvent, AppleEvent *reply, UInt32 refCon)
#else
pascal	OSErr	doOpenDoc(AppleEvent *theAppleEvent, AppleEvent *reply, long refCon)
#endif
{
	#pragma unused(reply, refCon)

	AEDescList	fileSpecList;
	FSSpec		file;
	OSErr		iErr = noErr;
	DescType		type;
	long			i,totalCount;
	Size			actual;
	AEKeyword	keyword;

	iErr = AEGetParamDesc(theAppleEvent, keyDirectObject, typeAEList, &fileSpecList);
	if(iErr ==noErr)
	{
		iErr = gotRequiredParams(theAppleEvent);
		if(iErr ==noErr)
			iErr = AECountItems(&fileSpecList, &totalCount);
		if(iErr ==noErr)
		{
			for(i=1;(i <= totalCount)&&(iErr == noErr);i++)
			{
				iErr = AEGetNthPtr(&fileSpecList, i, typeFSS, &keyword, &type, 
					(Ptr)&file, sizeof(FSSpec), &actual);

				if(iErr == noErr)
				{
					doOpenSpec((FSSpecPtr)&file);
				}
			}
		}
		iErr = AEDisposeDesc(&fileSpecList);
	}

	
	return(iErr);	
}
#ifdef TARGET_API_MAC_CARBON
OSErr	gotRequiredParams(const AppleEvent *appleEventPtr)
#else
OSErr	gotRequiredParams(AppleEvent *appleEventPtr)
#endif
{
	DescType	returnedType;
	Size	actualSize;
	OSErr	err;

	err = AEGetAttributePtr(appleEventPtr, keyMissedKeywordAttr, 
			typeWildCard, &returnedType, nil, 0, &actualSize);
	if(err == errAEDescNotFound)
		return(noErr);
	else if (err == noErr)
		return(errAEEventNotHandled);
	else
		return(err);
}

#ifdef TARGET_API_MAC_CARBON
pascal OSErr doPrintDoc(const AppleEvent *theAppleEvent, AppleEvent *reply, UInt32 refCon)
#else
pascal	OSErr	doPrintDoc(AppleEvent *theAppleEvent, AppleEvent *reply, long refCon)
#endif
{
	#pragma unused(theAppleEvent, reply, refCon)

	return(noErr);
}

#ifdef TARGET_API_MAC_CARBON
pascal OSErr doQuitApp(const AppleEvent *theAppleEvent, AppleEvent *reply, UInt32 refCon)
#else
pascal	OSErr	doQuitApp(AppleEvent *theAppleEvent, AppleEvent *reply, long refCon)
#endif
{
	#pragma unused(theAppleEvent, reply, refCon)

	gDone = true;

	return(noErr);
}

#ifdef TARGET_API_MAC_CARBON
pascal OSErr doShowPref(const AppleEvent *theAppleEvent, AppleEvent *reply, UInt32 refCon)
{
    #pragma unused(theAppleEvent, reply, refCon)
    WindowRef theWindow;
        
    theWindow = GetDialogWindow(showMyDialog(rPrefDLG));
    SelectWindow(theWindow);
        
    return(noErr);
}

void disposeAEEventHandler(void)
{
    DisposeAEEventHandlerUPP(doOpenAppHandler);
    DisposeAEEventHandlerUPP(doOpenDocHandler);
    DisposeAEEventHandlerUPP(doPrintDocHandler);
    DisposeAEEventHandlerUPP(doQuitAppHandler);
    DisposeAEEventHandlerUPP(doShowPrefHandler);
}
#endif
