/*	doGame	*/
#include	"applicationGlobals.h"

#ifdef TARGET_API_MAC_CARBON
extern prefVal	gPrefVal;				//globals for pref
#endif

#ifndef TARGET_API_MAC_CARBON
#pragma options align=mac68k
#endif

typedef struct {
	Ptr		mapNow;
	Ptr		mapBak;
	Ptr		mapTemp;
	short	putCount;		//count of put block
	short	startX;		//start X of block
	short	startY;		//start Y of block
	short	mapX;		//X value of map size
	short	mapY;		//Y value of map size
	short	minC;		//minimal X value of put block
	short	maxC;		//max X value of put block
} newMapRec, *newMapPeek;

#ifndef TARGET_API_MAC_CARBON
#pragma options align=reset
#endif

#define	kLoopMax		20
#define	kBlockRange	3
#define	kIncertColumn			100
#define	kIncertColumnPercentage	70

#define	kPutAtEnd				100
#define	kPutAtEndPercentage	40

#define	kMaxTumeStage	9

#define	kTumePutX		78
#define	kTumePutY		79
#define	kTumeCount		80


//#define	MAP_NOW(mapPtr, x, y)		(*((char *)((mapPtr) + (unsigned long)x + (unsigned long)kMapH*(unsigned long)(y))))

void		clearMap(const Ptr mapPtr);
void		makeRandomeMap(WindowPtr theWindow);
short	randomize(short range);
void		makeMap(WindowPtr theWindow);
long		getMaxCaracterCount(WindowPtr theWindow);
void		getTumeMapXY(WindowPtr theWindow);

Boolean	isThereBlank(newMapRec *newMap);
short	getIsolatedBlank(newMapRec *newMap);
short	getNextPos(newMapRec *newMap);
Boolean	putBlock(newMapRec *newMap);
Boolean	putOnePiece(newMapRec *newMap, short putX, short putY, unsigned char putData);
short	getPutPosY(newMapRec *newMap, short putX, unsigned char putData);
void		getColumnRange(newMapRec *newMap);
void		makeDeviation(WindowPtr theWindow);

unsigned char		randomeMap(void);
unsigned char		katayoriRand(void);
unsigned char		setStartPoint(newMapRec *newMap);
unsigned char		getPutData(newMapRec *newMap, short putX, short putY);

#ifdef TARGET_API_MAC_CARBON
extern WindowDataPtr getWindowDataPtr(WindowRef winodw);
extern DialogDataPtr getDialogDataPtr(DialogRef dialog);
#endif


void	drawAllCharactersOnWindow(WindowPtr wp)
{
#ifndef TARGET_API_MAC_CARBON
	OSErr	iErr	=	noErr;
#endif
	short	x,y;
	unsigned char		mapData;
	GrafPtr	savePort;

	if(wp == nil)
		return;

	GetPort(&savePort);
        
#ifdef TARGET_API_MAC_CARBON
	SetPortWindowPort(wp);
#else
	SetPort(wp);
#endif

	for(y = 1;y<=GAME_VAL(wp,mapY);y++)
	{
		for(x = 1;x<=GAME_VAL(wp,mapX);x++)
		{
			mapData = MAP_DATA(wp, x, y);
			drawOnePiece(wp,x,y,mapData);
		}
	}

	SetPort(savePort);
	return;
}


OSErr	newGame(WindowPtr theWindow)
{
	OSErr	iErr	=	noErr;
	GrafPtr	savePort;
        
#ifdef TARGET_API_MAC_CARBON	
        RgnHandle visRgn;
#endif
        
	if(theWindow == nil)
		return(iErr);

	//back up map
	if(GAME_VAL(theWindow, canGameUndo))
		setUndoBuffer(theWindow);
	
	//make map
	switch(GAME_VAL(theWindow, gameID))
	{
		case	rSame:
		case	rMame:
		case	rKome:
		case	rChoi:
		case	rYori:
			makeMap(theWindow);
		break;
		case	rTume:
			setGameBoardSize(theWindow);
			setWindowDisplayParts(theWindow);
			makeTumeGameMap(theWindow);
		break;
	}
	BlockMoveData(WIN_DATA(theWindow, mapData),WIN_DATA(theWindow, mapDataOrg),kMapMemorySize);

	//playing data
	GAME_VAL(theWindow, gameStatus) = kNoFlag;
	GAME_VAL(theWindow, tumeStage) = 0;
	GAME_VAL(theWindow, remain) = 0;
	GAME_VAL(theWindow, score) = 0;
	GAME_VAL(theWindow, bonus) = 0;
	GAME_VAL(theWindow, get) = 0;
	//
	//display data
	WIN_DATA(theWindow, clusterCount) = 0;
	//

	ShowWindow(theWindow);

	GetPort(&savePort);
#ifdef TARGET_API_MAC_CARBON
	SetPortWindowPort(theWindow);
        visRgn = NewRgn();
        GetPortVisibleRegion(GetWindowPort(theWindow), visRgn);
        InvalWindowRgn(theWindow, visRgn);
#else
	SetPort(theWindow);
	InvalRgn(theWindow->visRgn);
#endif
	SetPort(savePort);
	
	adjustMenuButton(theWindow);

	return(iErr);
}

Boolean	tumeNextStage(WindowPtr wp)
{	//return true when game is end
	Boolean	gameEnd = false;
	GrafPtr	savePort;
	long		cCount[6];
	Str255	mes1, mes2;
        
#ifdef TARGET_API_MAC_CARBON	
        RgnHandle visRgn;
#endif
        
	if(countCharactors(wp, cCount) != 0)		//when map not cleared
		gameEnd = true;					//game end

	if(!gameEnd)		//
	{	
		//calculate bonus and score
		if((GAME_VAL(wp, gameStatus) & kTumeSolution)!=0)
		{
			GAME_VAL(wp, bonus) = 0;
			GAME_VAL(wp, score) = 0;
		}
		else
			GAME_VAL(wp, bonus) += GAME_VAL(wp, mapX)*GAME_VAL(wp, mapY);
		//count up stage
		GAME_VAL(wp, tumeStage) += 1;
		if(GAME_VAL(wp, tumeStage) > kMaxTumeStage)
			gameEnd = true;		//game end
	}

	if(!gameEnd)		//
	{
		//increase game board size
		GAME_VAL(wp, mapX) = kMapTumeX + GAME_VAL(wp, tumeStage);
		if(GAME_VAL(wp, mapX)>kMaxMapX)
			GAME_VAL(wp, mapX) = kMaxMapX;
		GAME_VAL(wp, mapY) = kMapTumeY + GAME_VAL(wp, tumeStage)/2;
		if(GAME_VAL(wp, mapY)>kMaxMapY)
			GAME_VAL(wp, mapY) = kMaxMapY;	
		//***************
		//next stage
		//***************
		GAME_VAL(wp, score) +=GAME_VAL(wp, bonus);
		NumToString((long)GAME_VAL(wp, bonus),mes1);
		NumToString((long)GAME_VAL(wp, tumeStage)+1,mes2);
		alertTwoSeconds(rDLOG_TumegameNext, mes1, mes2, nil,nil);
		//playing data
		GAME_VAL(wp, gameStatus) = kNoFlag;
		GAME_VAL(wp, canGameUndo) = false;
		GAME_VAL(wp, remain) = 0;
		GAME_VAL(wp, get) = 0;
		//display data
		WIN_DATA(wp, clusterCount) = 0;
		//
		setWindowDisplayParts(wp);
		makeTumeGameMap(wp);

		ShowWindow(wp);
		SelectWindow(wp);

		GetPort(&savePort);
                
#ifdef TARGET_API_MAC_CARBON
		SetPortWindowPort(wp);
                visRgn = NewRgn();
                GetPortVisibleRegion(GetWindowPort(wp), visRgn);
                InvalWindowRgn(wp, visRgn);
#else
		SetPort(wp);
		InvalRgn(wp->visRgn);
#endif
		SetPort(savePort);
	}		

	return(gameEnd);		//return true when game is end
}

void		doMonkeyMode(void)
{
	WindowPtr	theWindow;
	unsigned long	secs;

	if(!isApplicationFront())
		return;

	if(!gPrefVal.isMonkeyMode)
		return;

	theWindow = findMyWindow();
	if(theWindow == nil)
		return;

	if(!GAME_STATUS(theWindow, kGameEnd))
		return;

	GetDateTime(&secs);
	if((secs - WIN_DATA(theWindow, gameEndSecs))<2)
		return;

	alertTwoSeconds(rDLOG_MonkeyNewGame, nil, nil, nil,nil);

	theWindow = findMyWindow();
	if(theWindow != nil)
	{
		GAME_VAL(theWindow, canGameUndo) = true;
		newGame(theWindow);
	}
}

#pragma mark -
//#define	MAP_NOW(mapPtr, x, y)		(*((char *)((mapPtr) + (unsigned long)x + (unsigned long)kMapH*(unsigned long)(y))))

unsigned char		getMapData(Ptr mapPtr, short x, short y)
{
	unsigned char	result = 0;
	
	if(x<0)
		bigBadError(eMapAxis);
	if(x>82)
		bigBadError(eMapAxis);
	if(y<0)
		bigBadError(eMapAxis);
	if(y>50)
		bigBadError(eMapAxis);

	result = *((unsigned char *)(mapPtr + (unsigned long)x + (unsigned long)kMapH*(unsigned long)y));
	
//	if(result ==1)	
//		result = 1;
	
	return(result);
}

void		setMapData(Ptr mapPtr, short x, short y, unsigned char mapData)
{
	if(x<0)
		bigBadError(eMapAxis);
	if(x>82)
		bigBadError(eMapAxis);
	if(y<0)
		bigBadError(eMapAxis);
	if(y>50)
		bigBadError(eMapAxis);

	*((unsigned char *)((mapPtr) + (unsigned long)x + (unsigned long)kMapH*(unsigned long)(y))) = mapData;
}

#pragma mark -

void		clearMap(const Ptr mapPtr)
{
	Ptr		p;

	p = mapPtr;
	while(p<=(mapPtr + kMapMemorySize))
	{
		*p = kMapBlank;
		p++;
	}
}

long		getMaxCaracterCount(WindowPtr theWindow)
{
	long		cCount[6];
	long		remain, cMax;
	short	i;

	remain = countCharactors(theWindow, cCount);
	cMax = 0;
	for(i = 1; i<6; i++)
	{
		if(cCount[i] > cMax)
			cMax = cCount[i];
	}

	return(cMax);
}

void		makeMap(WindowPtr theWindow)
{
	Ptr		pNew;
	long		cMax = 0, cNow;
	short	i, loopCount;

	clearMap(WIN_DATA(theWindow, mapData));

	//set loopCount from aimHSSelect
	if(gPrefVal.aimHSSelect == kAimHSLv1)
		loopCount = kAimHSLoop1;				//3 times
	else if(gPrefVal.aimHSSelect == kAimHSLv2)
		loopCount = kAimHSLoop2;				//5 times
	else if(gPrefVal.aimHSSelect == kAimHSLv3)
		loopCount = kAimHSLoop3;				//10 times
	else
		loopCount = kAimHSLoop0;				//once
		
	//make map once
	if(loopCount == kAimHSLoop0)
		makeRandomeMap(theWindow);
	else
	{	
		pNew = NewPtr(kMapMemorySize);
		showOSError(nil,  errorMemoryManager, MemError(), kFatal);

		//make map for loop count times
		for(i=0; i<loopCount; i++)
		{
			makeRandomeMap(theWindow);
			
			//if YoriGame, make deviation
			if(GAME_VAL(theWindow, gameID) == rYori)
				makeDeviation(theWindow);
			
			//count up largest number of kind 
			cNow = getMaxCaracterCount(theWindow);
			if(cNow > cMax)
			{
				cMax = cNow;
				//remenber map
				BlockMoveData(WIN_DATA(theWindow, mapData),pNew,kMapMemorySize);
			}

		}

		//dicide to most deviated map
		BlockMoveData(pNew,WIN_DATA(theWindow, mapData),kMapMemorySize);

		DisposePtr(pNew);			
	}

}

void		makeRandomeMap(WindowPtr theWindow)
{
	long		x,y;

	//make new randome map
	for(y = 1;y<=GAME_VAL(theWindow, mapY);y++)
	{
		for(x = 1;x<=GAME_VAL(theWindow,mapX);x++)
		{
			MAP_DATA(theWindow, x, y) = randomeMap();
		}
	}
}

void		makeDeviation(WindowPtr theWindow)
{
	long		x,y;
	unsigned char	c;
	short	r, ratio;

	c = randomeMap();			//character to deviate
	ratio = 20 + randomize(50);	//deviation ratio

	//change some character to 'c'
	for(y = 1;y<=GAME_VAL(theWindow, mapY);y++)
	{
		for(x = 1;x<=GAME_VAL(theWindow,mapX);x++)
		{
			r = randomize(500);
			if(r<ratio)	
				MAP_DATA(theWindow, x, y) = c;	//change it
		}
	}
}

#pragma mark -

Boolean	showTumeSolution(WindowPtr wp)
{
	Boolean		result = true;
	unsigned char	flag, count;
	short		x,y;

	if((GAME_VAL(wp, gameStatus) & kTumeSolution)==0)
		return(false);

	count = getMapData(WIN_DATA(wp, mapData),kTumeCount, 1);
	flag = getMapData(WIN_DATA(wp, mapData),kTumeCount, 2);
	
	x = getMapData(WIN_DATA(wp, mapData),kTumePutX, count);
	y = getMapData(WIN_DATA(wp, mapData),kTumePutY, count);

	if(flag == 0)
	{
		openPiece(wp,x,y);
		setMapData(WIN_DATA(wp, mapData),  kTumeCount, 2 , 1);
	}
	else
	{
		takePiece(wp, x,y);
		packPiece(wp);

		WIN_DATA(wp, drawIt) = true;
		WIN_DATA(wp, startY) = y;
		WIN_DATA(wp, drawY) = y;

		setMapData(WIN_DATA(wp, mapData),  kTumeCount, 1 , (unsigned char)(count-1));
		setMapData(WIN_DATA(wp, mapData),  kTumeCount, 2 , 0);
	}

	return(result);
}

Boolean	setTumeSolution(void)
{
	Boolean	result = false;
	short	itemHit;
        
	WindowPtr	wp;
	
#ifdef TARGET_API_MAC_CARBON	
        RgnHandle visRgn;
#endif
        
	wp = findMyWindow();
	if(wp == nil)
		return(result);

	if(GAME_VAL(wp, gameID)!=rTume)
		return(result);

	//alert: the score will be cleared
	setMyCursor(kArrowCURS);
	itemHit = Alert(rTumeGameSolution, nil);

	//if OK, show solution
	if(itemHit == ok)
	{
		result = true;
		GAME_VAL(wp, canGameUndo) = false;

		//original map
		BlockMoveData(WIN_DATA(wp, mapDataOrg),WIN_DATA(wp, mapData),kMapMemorySize);

		GAME_VAL(wp, gameStatus) = kNoFlag;
		GAME_VAL(wp, score) = 0;				//clear score
		GAME_VAL(wp, get) = 0;
		//
		closePiece(wp);
#ifdef TARGET_API_MAC_CARBON
                visRgn = NewRgn();
                GetPortVisibleRegion(GetWindowPort(wp), visRgn);
                InvalWindowRgn(wp, visRgn);
#else
		InvalRgn(wp->visRgn);
#endif
		//
		GAME_VAL(wp, gameStatus) |= kTumeSolution;
		//
		setMapData(WIN_DATA(wp, mapData),  kTumeCount, 1 , getMapData(WIN_DATA(wp, mapData),kTumeCount, 0));
		setMapData(WIN_DATA(wp, mapData),  kTumeCount, 2 , 0);
	}

	return(result);
}

#pragma mark -

Boolean	isThereBlank(newMapRec *newMap)
{
	Boolean	result = false;
	short	x,y;
	unsigned char		mapData;

	for(y = 1;y<=newMap->mapY;y++)
	{
		for(x = 1;x<=newMap->mapX;x++)
		{
			mapData = getMapData(newMap->mapNow, x, y);
			if((mapData&kMapCaraMask) == 0)		//now pos chara is not piece
				result = true;		//there is blank
		}
	}

	return(result);
}

unsigned char		getPutData(newMapRec *newMap, short putX, short putY)
{
	unsigned char	map[kNumOfKind+1];
	unsigned char	putData = kMapBlank;
	short	loopCount;
	
	map[0] = 0;
	map[1] = 0;
	map[2] = 0;
	map[3] = 0;
	map[4] = 0;
	map[5] = 0;

	//get surrounding piece
	map[(getMapData(newMap->mapNow, putX+1, putY)&kMapCaraMask)] ++;
	map[(getMapData(newMap->mapNow, putX-1, putY)&kMapCaraMask)] ++;
	map[(getMapData(newMap->mapNow, putX, putY)&kMapCaraMask)] ++;
	map[(getMapData(newMap->mapNow, putX, putY-1)&kMapCaraMask)] ++;

	loopCount = randomize(kNumOfKind*2)+1;
	putData = 1;
	while(loopCount>0)
	{
		putData++;
		if(putData>kNumOfKind)
			putData = 1;			

		while(map[putData] !=0)
		{
			putData++;
			if(putData>kNumOfKind)
				putData = 1;			
		}
		loopCount--;
	}

	return(putData);
}

Boolean	putOnePiece(newMapRec *newMap, short putX, short putY, unsigned char putData)
{	//return value:true : put succeed
	//                    false :put failed
	Boolean	result = false;
	short	y;
	unsigned char		mapData, tempData;

	tempData = putData;
	for(y = putY;y<=newMap->mapY;y++)
	{
		mapData = getMapData(newMap->mapNow,putX, y);
		if((mapData&kMapChecked) != 0)		//now pos chara is  checked
		{
			;	//next y		
		}
		else
		{
			setMapData(newMap->mapNow,putX, y, tempData);
			tempData = mapData;
		}
	}
	
	if(mapData != kMapBlank)					//if top piece is not blank
		return(result);						//put error
	
	result = true;
	for(y = putY;y<=newMap->mapY;y++)
	{
		mapData = getMapData(newMap->mapNow,putX, y);
		if((mapData&kMapChecked) != 0)		//now pos chara is  checked
		{
			mapData &= kMapUnCheck;
			if(mapData == getMapData(newMap->mapNow, putX+1, y))
				result = false;
			if(mapData == getMapData(newMap->mapNow, putX-1, y))
				result = false;
			if(mapData == getMapData(newMap->mapNow, putX, y+1))
				result = false;
			if(mapData == getMapData(newMap->mapNow, putX, y-1))
				result = false;
		}
		else
		{
			mapData |= kMapChecked;
			if(mapData == getMapData(newMap->mapNow, putX+1, y))
				result = false;
			if(mapData == getMapData(newMap->mapNow, putX-1, y))
				result = false;
			if(mapData == getMapData(newMap->mapNow, putX, y+1))
				result = false;
			if(mapData == getMapData(newMap->mapNow, putX, y-1))
				result = false;
		}
	}
	
	return(result);
}

unsigned char		setStartPoint(newMapRec *newMap)
{
#ifndef TARGET_API_MAC_CARBON
	short	loopDone = false;
#endif
	short	x,y;
	short	putX, putY;
#ifdef TARGET_API_MAC_CARBON
	short	columnCount;
#else
	short	columnCount, row = 0;
#endif
	unsigned char		mapData, putData;
	short	maxNum, rndNum;

	putData = kMapBlank;
	if(!isThereBlank(newMap))
		return(putData);

	//*************************************
	//count columns
	//*************************************
	columnCount = 0;
	for(x = 1;x<=newMap->mapX;x++)
	{
		mapData = getMapData(newMap->mapNow, x, 1);
		if((mapData&kMapCaraMask) != 0)		//now pos chara is piece
			columnCount++;
	}
	//*************************************
	//incert colomn
	//*************************************
	if((columnCount == 0)||((columnCount<newMap->mapX)&&(randomize(kIncertColumn)<kIncertColumnPercentage)))		//randomize() is ratio of colomn incert
	{
		putX = randomize(columnCount)+1;
		//incert colomn
		for(x = newMap->mapX;x>putX;x--)
		{
			for(y = 1;y<=newMap->mapY;y++)
			{
				mapData = getMapData(newMap->mapNow, x-1, y);
				setMapData(newMap->mapNow, x, y, mapData);
			}
		}
		for(y = 1;y<=newMap->mapY;y++)
		{
			setMapData(newMap->mapNow, putX, y, kMapBlank);
		}
		//put data
		putY = 1;
		putData = getPutData(newMap, putX, putY);
		if(putOnePiece(newMap, putX, putY, (unsigned char)(putData|kMapChecked)))
		{
			newMap->startX = putX;
			newMap->startY = putY;
			return(putData);
		}
		else
			return(kMapBlank);		//this will not happen
	}
	
	//*************************************
	//put piece at column with blank space
	//*************************************
	//find blank column
	maxNum = 0;
	for(x = 1;x<=columnCount;x++)
	{
		mapData = getMapData(newMap->mapNow, x, newMap->mapY);
		if(mapData == kMapBlank)		//now pos chara is blank
			maxNum ++;
	}
	if(maxNum == 0)
		return(kMapBlank);			//no blank space, can't put
	rndNum = randomize(maxNum)+1;
	for(x = 1;x<=columnCount;x++)
	{
		mapData = getMapData(newMap->mapNow, x, newMap->mapY);
		if(mapData == kMapBlank)		//now pos chara is blank
		{
			rndNum --;
			if(rndNum == 0)
			{
				putX = x;
				break;
			}
		}
	}
	//set Y position
	maxNum = 0;
	for(y = 1;y<=newMap->mapY;y++)
	{
		mapData = getMapData(newMap->mapNow, putX, y);
		if((mapData&kMapCaraMask) != 0)		//now pos chara is piece
			maxNum ++;
	}
	if(randomize(2)<1)
		putY = randomize(maxNum/2)+1;
	else
		putY = randomize(maxNum+1)+1;
	//put data
	putData = getPutData(newMap, putX, putY);
	if(putOnePiece(newMap, putX, putY, (unsigned char)(putData|kMapChecked)))
	{
		newMap->startX = putX;
		newMap->startY = putY;
		return(putData);
	}
	else
		return(kMapBlank);		//this will not happen

	return(putData);
}


short	getIsolatedBlank(newMapRec *newMap)
{
	short	x;
	short	putX = 0;
	
	//find isolated blank
	for(x=newMap->minC;x<=newMap->maxC;x++)
	{
		if((getMapData(newMap->mapNow, x, newMap->mapY) == kMapBlank)&&
			(getMapData(newMap->mapNow, x-1, newMap->mapY) != kMapBlank)&&
			(getMapData(newMap->mapNow, x+1, newMap->mapY) != kMapBlank))
		{
			putX = x;
			break;
		}
	}
	
	return(putX);
}

short	getNextPos(newMapRec *newMap)
{
	short	x;
	short	loopCount;
	short	putX = 0;

	//*************************************
	//put piece at right or left end of the block already put
	//*************************************
	if(getMapData(newMap->mapNow, newMap->minC, newMap->mapY) == kMapBlank)
	{
		if(randomize(kPutAtEnd)<kPutAtEndPercentage)
			return(newMap->minC);
	}
	
	if(getMapData(newMap->mapNow, newMap->maxC, newMap->mapY) == kMapBlank)
	{
		if(randomize(kPutAtEnd)<kPutAtEndPercentage)
			return(newMap->maxC);
	}
	//*************************************
	//put piece at random at the block already put
	//*************************************
	x = newMap->minC + randomize(newMap->maxC - newMap->minC);
	loopCount = newMap->maxC - newMap->minC + 1;
	while(loopCount>0)
	{
		x ++;
		if(x > newMap->maxC)
			x = newMap->minC;
		if(getMapData(newMap->mapNow,x, newMap->mapY) == kMapBlank)
		{
			putX = x;
			break;
		}
		loopCount--;
	}

	return(putX);
}

short	getPutPosY(newMapRec *newMap, short putX, unsigned char putData)
{
	Boolean	canPutY[kMaxMapY+1];
	short 	putY = 0;
	short	y;
	short	loopCount;
	Boolean	canPut = false;

	for(y=0; y<= kMaxMapY; y++)
	{
		canPutY[y] = false;
	}

	for(y=1; y<= newMap->mapY; y++)
	{

		if(getMapData(newMap->mapNow,putX, y) == (putData|kMapChecked))
			canPutY[y] = false;
		else
			canPutY[y] = true;

		if((y>1)&&(getMapData(newMap->mapNow,putX, y-1) == kMapBlank))
			canPutY[y] = false;		

		//next pos must checked
		if(canPutY[y])	
		{
			if(getMapData(newMap->mapNow,putX, y+1) == (putData|kMapChecked))
				;
			else if(getMapData(newMap->mapNow,putX, y-1) == (putData|kMapChecked))
				;
			else if(getMapData(newMap->mapNow,putX+1, y) == (putData|kMapChecked))
				;
			else if(getMapData(newMap->mapNow,putX-1, y) == (putData|kMapChecked))
				;
			else
				canPutY[y] = false;
		}
		//next pos must not same
		if(canPutY[y])	
		{
			if(getMapData(newMap->mapNow,putX, y) == putData)
				canPutY[y] = false;
			else if(getMapData(newMap->mapNow,putX, y-1) == putData)
				canPutY[y] = false;
			else if(getMapData(newMap->mapNow,putX+1, y) == putData)
				canPutY[y] = false;
			else if(getMapData(newMap->mapNow,putX-1, y) == putData)
				canPutY[y] = false;
		}
		if(canPutY[y])	
			canPut = true;
	}

	if(canPut)
	{
		y = randomize(newMap->mapY);
		loopCount = randomize(newMap->mapY) + 1;
		while(loopCount>0){
			y++;
			if(y > newMap->mapY)
				y = 1;

			while(!canPutY[y]){
				y++;
				if(y > newMap->mapY)
					y = 1;
			}
			loopCount--;
		}
		
		putY = y;
	}

	return(putY);
}
void		getColumnRange(newMapRec *newMap)
{
	short	x, y;
	unsigned char		mapData;

	newMap->minC = newMap->mapX;
	newMap->maxC = 1;

	for(x = 1; x <= newMap->mapX; x++)
	{
		for(y = 1;y<=newMap->mapY;y++)
		{
			mapData = getMapData(newMap->mapNow, x, y);
			if((mapData&kMapChecked)!=0)				//map is checked
			{
				if(x<newMap->minC)
					newMap->minC = x;

				if(x>newMap->maxC)
					newMap->maxC = x;			
			}
		}
	}
	
	newMap->minC--;
	if(newMap->minC<1)
		newMap->minC = 1;

	newMap->maxC++;
	if(newMap->maxC>newMap->mapX)
		newMap->maxC = newMap->mapX;

	//below will never happen
	if(newMap->minC == newMap->mapX)
		newMap->minC = 1;
	if(newMap->maxC == 1)
		newMap->maxC = newMap->mapX;
}

Boolean	putBlock(newMapRec *newMap)
{
	Boolean	result = false;
	short	blockCount, maxBlock;
	short	loopCount;
	unsigned char		putData;
	short	putX, putY;

	maxBlock = 2 + randomize(kBlockRange);
	if(maxBlock>kBlockRange)
		maxBlock += randomize(kBlockRange);

	//put block
	putData = setStartPoint(newMap);
	if(putData != kMapBlank)
	{
		BlockMoveData(newMap->mapNow,newMap->mapTemp, kMapMemorySize);	//save map

		blockCount = 1;
		loopCount = 0;
		while(loopCount<kLoopMax)
		{
			getColumnRange(newMap);
			putY = 0;
			//search isolated blank
			putX = getIsolatedBlank(newMap);
			if(putX != 0)
				putY = getPutPosY(newMap, putX, putData);
			//search blank
			if(putY == 0)
			{
				if(blockCount<maxBlock)
				{
					putX = getNextPos(newMap);
					if(putX != 0)
						putY = getPutPosY(newMap, putX, putData);
				}
				else
				{
					loopCount = kLoopMax;
					break;
				}
			}
			//put one piece
			if(putY != 0)
			{
				BlockMoveData(newMap->mapNow, newMap->mapTemp, kMapMemorySize);	//save map
				result = putOnePiece(newMap, putX, putY, (unsigned char)(putData|kMapChecked));
				if(!result)
					BlockMoveData(newMap->mapTemp, newMap->mapNow, kMapMemorySize);	//restore map
			}
			else
				result = false;
			//count up
			if(result)
			{
				blockCount ++;
				loopCount = 0;
			}
			else
			{
				loopCount++;
			}
		}
	}

	if(result&&(newMap->putCount<kMaxMapY))
	{	//put success then record the position
		setMapData(newMap->mapNow,  kTumePutX, newMap->putCount, (unsigned char)(newMap->startX));
		setMapData(newMap->mapNow, kTumePutY, newMap->putCount, (unsigned char)(newMap->startY));
		setMapData(newMap->mapNow,  kTumeCount, 0 , (unsigned char)(newMap->putCount));
		newMap->putCount ++;
	}

	return(result);
}

void		makeTumeGameMap(WindowPtr theWindow)
{
	newMapRec	newMap;
	short	loopCount;
	Boolean	result = true;
	short	x,y;
	unsigned char mapData;
	
	//allocate memory for making map
	newMap.mapBak = NewPtrClear(kMapMemorySize);
	showOSError(nil,  errorMemoryManager, MemError(), kFatal);
	newMap.mapTemp = NewPtrClear(kMapMemorySize);
	showOSError(nil,  errorMemoryManager, MemError(), kFatal);
	//
	newMap.mapNow = WIN_DATA(theWindow, mapData);
	//
	newMap.mapX = GAME_VAL(theWindow, mapX);
	newMap.mapY = GAME_VAL(theWindow, mapY);
	newMap.putCount = 1;
	//
	loopCount = 0;
	clearMap(WIN_DATA(theWindow, mapData));
	BlockMoveData(newMap.mapNow, newMap.mapBak, kMapMemorySize);		//back up map
	while(loopCount< kLoopMax)
	{
		result = putBlock(&newMap);
		if(result)
		{	//put success
			//uncheck and clean map data
			for(y = 1;y<=newMap.mapY;y++)
			{
				for(x = 1;x<=newMap.mapX;x++)
				{
					mapData = getMapData(newMap.mapNow,  x,  y);
					setMapData(newMap.mapNow,  x,  y, (unsigned char)(mapData&kMapUnCheck));
				}
			}
			//store map to backup
			BlockMoveData(newMap.mapNow, newMap.mapBak, kMapMemorySize);
			loopCount = 0;
		}
		else
		{	//put failed
			//restore map from backup
			BlockMoveData(newMap.mapBak, newMap.mapNow, kMapMemorySize);	//restore map
			//try putBlock() again
			loopCount++;
		}
	}

	BlockMoveData(newMap.mapBak, newMap.mapNow, kMapMemorySize);	//final map
	BlockMoveData(WIN_DATA(theWindow, mapData),WIN_DATA(theWindow, mapDataOrg),kMapMemorySize);	//remenber original
	//release memory
	DisposePtr(newMap.mapBak);			
	DisposePtr(newMap.mapTemp);			
}

#pragma mark -

short	randomize(short range)
{	//0<= return < range
	#define	kRandomUpperLimit	32768

	long	randomNumber;
	
	randomNumber = Random();
	if(randomNumber < 0)
		randomNumber *= -1;
	return((short)((randomNumber * range)/kRandomUpperLimit));

	#undef	kRandomUpperLimit
}


unsigned char	randomeMap(void)
{
	short	r;
	unsigned char		num;
	
	r = randomize(500);

	if(r<100)
		num = 1;
	else if(r<200)
		num = 2;
	else if(r<300)
		num = 3;
	else if(r<400)
		num = 4;
	else
		num = 5;

	return(num);
}

unsigned char	katayoriRand(void)		//	0<= return <5	
{
	short	katayori = 0;
	//
	short	r;
	unsigned char	num;
	
	r = randomize(500);

	if(r<(100+katayori*2))
		num = 1;
	else if(r<(200+katayori*3))
		num = 2;
	else if(r<(300+katayori*3))
		num = 3;
	else if(r<(400+katayori*2))
		num = 4;
	else
		num = 5;

	return(num);
}

#pragma mark -

void	getTumeMapXY(WindowPtr theWindow)
{
	#pragma unused(theWindow)
}

Boolean	setGameBoardSize(WindowPtr theWindow)
{
	Boolean	sizeChanged = true;

	switch(GAME_VAL(theWindow, gameID))
	{
		case	rSame:
			GAME_VAL(theWindow, mapX) = (short)kMapSameX;
			GAME_VAL(theWindow, mapY) = (short)kMapSameY;
		break;
		case	rMame:
			GAME_VAL(theWindow, mapX) = (short)kMapMameX;
			GAME_VAL(theWindow, mapY) = (short)kMapMameY;
		break;
		case	rKome:
			GAME_VAL(theWindow, mapX) = (short)kMapKomeX;
			GAME_VAL(theWindow, mapY) = (short)kMapKomeY;
		break;
		case	rChoi:
			GAME_VAL(theWindow, mapX) = (short)kMapChoiX;
			GAME_VAL(theWindow, mapY) = (short)kMapChoiY;
		break;
		case	rTume:
			getTumeMapXY(theWindow);
			GAME_VAL(theWindow, mapX) = (short)kMapTumeX;
			GAME_VAL(theWindow, mapY) = (short)kMapTumeY;
		break;
		case	rYori:
			GAME_VAL(theWindow, mapX) = (short)kMapYoriX;
			GAME_VAL(theWindow, mapY) = (short)kMapYoriY;
		break;
	}

	return(sizeChanged);
}

short gameID2MenuID(short gameID)
{
	short	menuID = 0;

	switch(gameID)
	{
		case	rSame:
			menuID = iSame;
		break;
		case	rMame:
			menuID = iMame;
		break;
		case	rKome:
			menuID = iKome;
		break;
		case	rChoi:
			menuID = iChoi;
		break;
		case	rTume:
			menuID = iTume;
		break;
		case	rYori:
			menuID = iYori;
		break;
	}

	return(menuID);

}

short	menuID2GameID(short menuItemNum)
{	//rID of the score
	short	gameID = 0;

	switch(menuItemNum)
	{
		case	iSame:
			gameID = rSame;
		break;
		case	iMame:
			gameID = rMame;
		break;
		case	iKome:
			gameID = rKome;
		break;
		case	iChoi:
			gameID = rChoi;
		break;
		case	iTume:
			gameID = rTume;
		break;
		case	iYori:
			gameID = rYori;
		break;
		default:
			gameID = rOther;
		break;
	}

	return(gameID);
}

void	setWindowDisplayParts(WindowPtr theWindow)
{
	PicHandle	hPic;
	Rect		theRect;
	short	hSize, vSize;
	MenuHandle	menuHandle;
	Str255	theString;
	GrafPtr	savePort;
	short	winH, winV;
	short	fRefNum;
        
#ifdef TARGET_API_MAC_CARBON
        BitMap screenBits;
        Rect portRect;
#endif

	if(theWindow == nil)
		return;

	GetPort(&savePort);
        
#ifdef TARGET_API_MAC_CARBON
	SetPortWindowPort(theWindow);
#else
	SetPort(theWindow);
#endif

	//set koma size
	if(gPrefVal.isAutoSize)
	{
#ifdef TARGET_API_MAC_CARBON
                GetQDGlobalsScreenBits(&screenBits);
                theRect = screenBits.bounds;
		vSize = (theRect.bottom - theRect.top - GetMBarHeight() - GetMBarHeight())/GAME_VAL(theWindow, mapY);
#else
		theRect = qd.screenBits.bounds;
		vSize = (theRect.bottom - theRect.top - LMGetMBarHeight() - LMGetMBarHeight())/GAME_VAL(theWindow, mapY);
#endif
		hSize = (theRect.right - theRect.left)/GAME_VAL(theWindow, mapX);
		if(vSize < hSize)
			GAME_VAL(theWindow, komaSize) = vSize;
		else
			GAME_VAL(theWindow, komaSize) = hSize;

		if(GAME_VAL(theWindow, komaSize) > kCharaSizeMax)
			GAME_VAL(theWindow, komaSize) = kCharaSizeMax;
		if(GAME_VAL(theWindow, komaSize) < 1)
			GAME_VAL(theWindow, komaSize) = 1;
	}
	else
		GAME_VAL(theWindow, komaSize) = gPrefVal.charaSize;

	//read PICT from preference, change the size of piece
	fRefNum = openPreferenceFile();
	if(fRefNum != kResFileNotOpened)
	{
		hPic = (PicHandle)GetResource(ftPICTFile,kKomaPICT);
		drawCharaPICTtoGWorld(theWindow, hPic);
		closePreferences(fRefNum);
	}
	else
	{	//this will not happen, because already duplicate PICT data to preference file
		hPic = (PicHandle)GetResource(ftPICTFile,kKomaPICT);
		drawCharaPICTtoGWorld(theWindow, hPic);
	}
	//set window title
	menuHandle = GetMenuHandle(mGameType);
	GetMenuItemText(menuHandle,gameID2MenuID(GAME_VAL(theWindow,gameID)),theString);		//GetItem		(old name)
	SetWTitle(theWindow, theString);

	//resize window
	winH = GAME_VAL(theWindow,mapX)*GAME_VAL(theWindow,komaSize);
	WIN_DATA(theWindow, scoreTop) = kButtonHeight;
	WIN_DATA(theWindow, thumbTop) = WIN_DATA(theWindow, scoreTop);
	WIN_DATA(theWindow, thumbH) = kHeaderThumbH;
	WIN_DATA(theWindow, clusterH) = 0;
	//calculate header height
	if(winH > 320)
	{
		WIN_DATA(theWindow, headerHeight) = kHeaderHeight + kButtonHeight;
	}
	else
	{
		WIN_DATA(theWindow, headerHeight) = kHeaderHeight*2 + kButtonHeight;
		WIN_DATA(theWindow, thumbTop) = kHeaderHeight + kButtonHeight;
		WIN_DATA(theWindow, thumbH) = 12;
	}
	//
	winV = GAME_VAL(theWindow,mapY)*GAME_VAL(theWindow,komaSize) + WIN_DATA(theWindow, headerHeight);	
#ifdef TARGET_API_MAC_CARBON
	SizeWindow(theWindow, winH, winV,true);
        GetPortBounds(GetWindowPort(theWindow), &portRect);
        EraseRect(&portRect);
#else
	SizeWindow(theWindow, winH, winV,true);
	EraseRect(&(theWindow->portRect));
#endif

	SetPort(savePort);
}

void	readHiScoreDataToWindowRec(short gameID, WindowPtr theWindow)
{
	short	fRefNum;

	fRefNum = openPreferenceFile();
	if(fRefNum != kResFileNotOpened)
	{
		DisposeHandle(WIN_DATA(theWindow, hiscoreData));
		showOSError(nil,  errorMemoryManager, MemError(), kFatal);
		WIN_DATA(theWindow, hiscoreData) = getHiScore(gameID, kRankScoreType);
		DetachResource(WIN_DATA(theWindow, hiscoreData));
		showOSError(nil,  errorResourceManager, ResError(), kFatal);
		//
		DisposeHandle(WIN_DATA(theWindow, hiscoreRemain));
		showOSError(nil,  errorMemoryManager, MemError(), kFatal);
		WIN_DATA(theWindow, hiscoreRemain) = getHiScore(gameID, kRankRemainType);
		DetachResource(WIN_DATA(theWindow, hiscoreRemain));
		showOSError(nil,  errorResourceManager, ResError(), kFatal);
		//
		closePreferences(fRefNum);
	}

}

void	doGameTypeMenu(short itemNum)
{
	WindowPtr	theWindow;
	Boolean		doChange = true;

	theWindow = findMyWindow();
	if(theWindow != nil)
	{
		if(GAME_VAL(theWindow, gameID) == menuID2GameID(itemNum))
			doChange = false;
			
		if(itemNum == iTume)
			doChange = true;
		
		if(doChange)
		{
			GAME_VAL(theWindow, gameID) = menuID2GameID(itemNum);
			GAME_VAL(theWindow, gameStatus) = kNoFlag;
			GAME_VAL(theWindow, canGameUndo) = false;
			setGameBoardSize(theWindow);
			setWindowDisplayParts(theWindow);
			moveCenterOfMainScreen(theWindow);
			newGame(theWindow);
		}
	}
}
